%
%
%  Camera Manager Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Camera Functions">
%        <Item name="Camera Manager" icon="Matlab" tooltip="Starts the camera manager.">
%          <Command>MatlabXT::XTCameraManager(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%    </CustomTools>
% 
%
%  Description:
%   
%   Save/load/clear camera settings in .mat files.
%   Settings are: Position, orientation, perspective, ortographic, 
%       focus and height.
%
%

function XTCameraManager(aImarisApplicationID)

if isempty(cell2mat(strfind(javaclasspath('-all'), 'ImarisLib.jar')))
  mlock
  javaaddpath ImarisLib.jar
  munlock
end

persistent vImarisApplication vEditFileName

% button callbacks
if aImarisApplicationID==-1
    return;
elseif isa(aImarisApplicationID, 'numeric') && aImarisApplicationID==-5;
    Browse(vEditFileName);
    return;
elseif isa(aImarisApplicationID, 'numeric') && aImarisApplicationID<-1;
    Execute(-aImarisApplicationID-1, vImarisApplication, vEditFileName);
    return;
end

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

% create the window
uicontrol('Style', 'text', 'String', 'Imaris Camera Manager', ...
    'FontName', 'Times New Roman', 'FontSize', 10, ...
    'Position', [10,70,170,20], 'DeleteFcn', 'XTCameraManager(-1)');
uicontrol('Style', 'pushbutton', 'String', 'Browse...', ...
    'Position', [210,70,80,20], 'Callback', 'XTCameraManager(-5)');
uicontrol('Style', 'text', 'String', 'File Name:', ...
    'FontName', 'Times New Roman', 'FontSize', 10, ...
    'Position', [10,50,70,20]);
vEditFileName = uicontrol('Style', 'edit', 'String', 'ImarisCamera.mat', ...
    'FontName', 'Times New Roman', 'FontSize', 10, ...
    'Position', [80,50,210,20]);

uicontrol('Style', 'pushbutton', 'String', 'Save', ...
    'Position', [10,10,80,30], 'Callback', 'XTCameraManager(-2)');
uicontrol('Style', 'pushbutton', 'String', 'Load', ...
    'Position', [110,10,80,30], 'Callback', 'XTCameraManager(-3)');
uicontrol('Style', 'pushbutton', 'String', 'Clear', ...
    'Position', [210,10,80,30], 'Callback', 'XTCameraManager(-4)');

vWindow = get(vEditFileName, 'Parent');
vWindowPosition = get(vWindow, 'Position');
vWindowPosition(3:4) = [300, 100];
set(vWindow, 'Position', vWindowPosition);


%----------------------------------------------%

function Browse(vEditFileName)

vFileName = get(vEditFileName,'String');
[vFileName, vPathName] = uiputfile({'*.mat','MAT-files (*.mat)'}, ...
    'Select the file', vFileName);
if ~isequal(vFileName,0)
    set(vEditFileName,'String',fullfile(vPathName,vFileName));
end


%----------------------------------------------%

function Execute(aOperation, vImarisApplication, vEditFileName)
% aOperation = 1 save, 2 load, 3 clear

vStringOperations = {'save', 'load', 'clear'};
vFileName = get(vEditFileName,'String');

vCamera = vImarisApplication.GetSurpassCamera;

if aOperation==1
    vName = inputdlg({'Please enter the camera settings name:'}, ...
        'Imaris Object Manager', 1, {'Default'});
    vName = char(vName);
    if isempty(vName)
        return;
    end
else
    % load the list of the saved camera settings
    vNumberOfObjects = 0;
    if exist(vFileName, 'file')
        vCameraNames = { };
        load(vFileName, 'vCameraNames');
        vNumberOfObjects = numel(vCameraNames);
    end
    if vNumberOfObjects == 0
        msgbox(['There is no object to ', vStringOperations{aOperation}, '!']);
        return;
    end
    
    [vSelectedObject, vOk] = listdlg('ListString', vCameraNames, ...
                           'SelectionMode','single',...
                           'ListSize',[250 150],...
                           'Name','Imaris Object Manager',...
                           'PromptString',{['Please select the object to ', ...
                           vStringOperations{aOperation}, ':']});
    if vOk<1, return; end;
    
    vName = char(vCameraNames(vSelectedObject));

end

% remove invalid characters
vName_ = vName; 
for vLetter = 1:length(vName)
    vChar = vName(vLetter);
    if (vChar<'0' || vChar>'9') && (vChar<'a' || vChar>'z') && (vChar<'A' || vChar>'Z')
        vName_(vLetter) = '_';
    end
end

% names (that should be unique) of the variables in the file.mat
vValidName = ['v',vName_];

if aOperation==1 % save

    if exist(vFileName, 'file')
        load(vFileName, 'vCameraNames');

        % if the variable is already saved, their values are overwritten
        % but no new object is added
        vFound = false;
        vNumberOfObjects = length(vCameraNames);
        for vIndex = 1:vNumberOfObjects
            if strcmp(vName,char(vCameraNames(vIndex)))
                vFound = true;
                break;
            end
        end
        if vFound
            vAnswer = questdlg(['Overwrite ',vName,'?'], ...
                'Imaris Object Manager', 'Yes', 'No', 'Yes');
            if ~strcmp(vAnswer,'Yes')
                return;
            end
        else
            vCameraNames{vNumberOfObjects+1} = vName;
            save(vFileName, 'vCameraNames', '-append');
        end
    else
        vCameraNames = {vName};
        save(vFileName, 'vCameraNames');   
    end
    
    vXYZ = vCamera.GetPosition;
    vOrientation = vCamera.GetOrientationAxisAngle;

    eval([vValidName,' = [vXYZ'', vOrientation.mAxisXYZ'', vOrientation.mAngle, ', ...
        'vCamera.GetPerspective, vCamera.GetFocus, vCamera.GetHeight];']);

    save(vFileName, vValidName, '-append');

elseif aOperation==2 % load

    load(vFileName, vValidName);
    vSettings = eval(vValidName);
    vCamera.SetPosition(vSettings(1:3));
    vCamera.SetOrientationAxisAngle(vSettings(4:6), vSettings(7));
    vCamera.SetPerspective(vSettings(8) == 1);
    vCamera.SetFocus(vSettings(9));
    vCamera.SetHeight(vSettings(10));
    
else % clear

    % copy the data into a struct
    % deletes the original file
    % rebuild the right file

    vAnswer = questdlg(['Remove ',vName,' from the memory?'], ...
        'Imaris Object Manager', 'Yes', 'No', 'Yes');
    if ~strcmp(vAnswer,'Yes')
        return;
    end
    
    vFile = load(vFileName);
    vFields = fieldnames(vFile);

    % remove the name from the lists
    vRemaining = [1:vSelectedObject-1,vSelectedObject+1:vNumberOfObjects];
    vCameraNames = vCameraNames(vRemaining);
    save(vFileName, 'vCameraNames');

    vNumberOfLetters = length(vValidName);
    for vVariable = 1:length(vFields)
        vString = char(vFields(vVariable));
        if ~strcmp(vString,'vCameraNames') && ...
                (length(vString) < vNumberOfLetters || ...
                ~strcmp(vValidName,vString))
            eval([vString,' = vFile.(vString);']);
            save(vFileName,vString,'-append');
            clear(vString);
        end
    end
end

